#include "oswrap.h"
#include "cridc.h"
#include "cridw_linux.h"
#include "cridw_pci.h"

/*
 * Try to find PCI BIOS.
 */

int pci_find_bios(void)
{
	union bios32 *check;
	unsigned char sum;
	int i, length;

#ifdef DEBUG 
	printk(KERN_DEBUG "finding bios32 entry point\n");
#endif
	/*
	 * Follow the standard procedure for locating the BIOS32 Service
	 * directory by scanning the permissible address range from
	 * 0xe0000 through 0xfffff for a valid BIOS32 structure.
	 */

	for (check = (union bios32 *) __va(0xe0000);
	     check <= (union bios32 *) __va(0xffff0);
	     ++check) {
		if (check->fields.signature != BIOS32_SIGNATURE)
			continue;
		length = check->fields.length * 16;
		if (!length)
			continue;
		sum = 0;
		for (i = 0; i < length ; ++i)
			sum += check->chars[i];
		if (sum != 0)
			continue;
		if (check->fields.revision != 0) {
			printk("PCI: unsupported BIOS32 revision %d at 0x%p, report to <mj@suse.cz>\n",
				check->fields.revision, check);
			continue;
		}
#ifdef DEBUG
		printk(KERN_DEBUG "PCI: BIOS32 Service Directory structure at 0x%p\n", check);
#endif
		if (check->fields.entry >= 0x100000) {
			printk("PCI: BIOS32 entry (0x%p) in high memory, cannot use.\n", check);
			return -1;
		} else {
			unsigned long bios32_entry = check->fields.entry;
#ifdef DEBUG
			printk(KERN_DEBUG "PCI: BIOS32 Service Directory entry at 0x%lx\n", bios32_entry);
#endif
			bios32_indirect.address = bios32_entry + PAGE_OFFSET;
			return 0;
		}
		break;	/* Hopefully more than one BIOS32 cannot happen... */
	}

	return -1;
}
/*
 * Returns the entry point for the given service, NULL on error
 */

static unsigned long bios32_service(unsigned long service)
{
	unsigned char return_code;	/* %al */
	unsigned long address;		/* %ebx */
	unsigned long length;		/* %ecx */
	unsigned long entry;		/* %edx */
	unsigned long flags;

#ifdef DEBUG
	printk(KERN_DEBUG "entering bios32_service\n");
#endif

	if (bios32_indirect.address == 0) {
		return_code = pci_find_bios();
		if (return_code != 0) return 0;
	}

	__save_flags(flags); __cli();
	__asm__("lcall *(%%edi); cld"
		: "=a" (return_code),
		  "=b" (address),
		  "=c" (length),
		  "=d" (entry)
		: "0" (service),
		  "1" (0),
		  "D" (&bios32_indirect));
	__restore_flags(flags);

	switch (return_code) {
		case 0:
			return address + entry;
		case 0x80:	/* Not present */
			printk(KERN_WARNING "bios32_service(0x%lx): not present\n", service);
			return 0;
		default: /* Shouldn't happen */
			printk(KERN_WARNING "bios32_service(0x%lx): returned 0x%x -- BIOS bug!\n",
				service, return_code);
			return 0;
	}
}

static int check_pcibios(void)
{
	u32 signature, eax, ebx, ecx;
	u8 status, major_ver, minor_ver, hw_mech;
	unsigned long flags, pcibios_entry;

#ifdef DEBUG
	printk(KERN_DEBUG "entering check_pcibios\n");
#endif

	if ((pcibios_entry = bios32_service(PCI_SERVICE))) {
		pci_indirect.address = pcibios_entry + PAGE_OFFSET;

		__save_flags(flags); __cli();
		__asm__(
			"lcall *(%%edi); cld\n\t"
			"jc 1f\n\t"
			"xor %%ah, %%ah\n"
			"1:"
			: "=d" (signature),
			  "=a" (eax),
			  "=b" (ebx),
			  "=c" (ecx)
			: "1" (PCIBIOS_PCI_BIOS_PRESENT),
			  "D" (&pci_indirect)
			: "memory");
		__restore_flags(flags);

		status = (eax >> 8) & 0xff;
		hw_mech = eax & 0xff;
		major_ver = (ebx >> 8) & 0xff;
		minor_ver = ebx & 0xff;
#ifdef DEBUG
		printk(KERN_DEBUG "PCI: BIOS probe returned s=%02x hw=%02x ver=%02x.%02x\n",
			status, hw_mech, major_ver, minor_ver);
#endif
		if (status || signature != PCI_SIGNATURE) {
			printk (KERN_ERR "PCI: BIOS BUG #%x[%08x] found\n",
				status, signature);
			return 0;
		}
		printk(KERN_INFO "PCI: PCI BIOS revision %x.%02x entry at 0x%lx\n",
			major_ver, minor_ver, pcibios_entry);
		return 1;
	}
	return 0;
}

struct irq_routing_table * pcibios_get_irq_routing_table(void)
{
	struct irq_routing_options opt;
	struct irq_routing_table *rt = NULL;
	int ret, map;
	unsigned long page;
	static int pci_bios_present;

#ifdef DEBUG
	printk(KERN_DEBUG "entering pcibios_get_irq_routing_table\n");
#endif

	if (!pci_bios_present) {
		pci_bios_present = check_pcibios();		
		if (!pci_bios_present) return NULL;
	}
	page = __get_free_page(GFP_KERNEL);
	if (!page)
		return NULL;
	opt.table = (struct irq_info *) page;
	opt.size = PAGE_SIZE;
	opt.segment = __KERNEL_DS;

#ifdef DEBUG
	printk(KERN_DEBUG "PCI: Fetching IRQ routing table...\n");
#endif
	__asm__("push %%es\n\t"
		"push %%ds\n\t"
		"pop  %%es\n\t"
		"lcall *(%%esi); cld\n\t"
		"pop %%es\n\t"
		"jc 1f\n\t"
		"xor %%ah, %%ah\n"
		"1:"
		: "=a" (ret),
		  "=b" (map)
		: "0" (PCIBIOS_GET_ROUTING_OPTIONS),
		  "1" (0),
		  "D" ((long) &opt),
		  "S" (&pci_indirect));
#ifdef DEBUG
	printk(KERN_DEBUG "OK  ret=%d, size=%d, map=%x\n", ret, opt.size, map);
#endif
	if (ret & 0xff00)
		printk(KERN_ERR "PCI: Error %02x when fetching IRQ routing table.\n", (ret >> 8) & 0xff);
	else if (opt.size) {
		rt = kmalloc(sizeof(struct irq_routing_table) + opt.size, GFP_KERNEL);
		if (rt) {
			memset(rt, 0, sizeof(struct irq_routing_table));
			rt->size = opt.size + sizeof(struct irq_routing_table);
			rt->exclusive_irqs = map;
			memcpy(rt->slots, (void *) page, opt.size);
			printk(KERN_INFO "PCI: Using BIOS Interrupt Routing Table\n");
		}
	}
	free_page(page);
	return rt;
}

int pci_get_slot(struct pci_dev* pdev, u8 * slot)
{
	struct irq_routing_table *pci_irq_routing_info_length;
	u32 work;

	int len;
	int loop;
	int rc = -1;

	u8 tbus, tdevfn, tslot, bridge_slot;

	bridge_slot = 0xFF;

	pci_irq_routing_info_length = pcibios_get_irq_routing_table();
	if (pdev == NULL) goto out;

	len = (pci_irq_routing_info_length->size -
	     sizeof(struct irq_routing_table)) / sizeof(struct irq_info);
	// Make sure I got at least one entry
	if (len == 0)  goto out;


	for (loop = 0; loop < len; ++loop) {
		tbus = pci_irq_routing_info_length->slots[loop].bus;
		tdevfn = pci_irq_routing_info_length->slots[loop].devfn;
		tslot = pci_irq_routing_info_length->slots[loop].slot;

#ifdef DEBUG	
		printk(KERN_DEBUG "%d. bus %d devfn %2.2x slot %d\n", loop, tbus, tdevfn, tslot);
#endif

		if ((tbus == pdev->bus->number) && (PCI_SLOT(tdevfn)== PCI_SLOT(pdev->devfn))) {
			*slot = tslot;
			rc = 0;
			goto out;

		} else {
// Didn't get a match on the target PCI device. Check if the
// current IRQ table entry is a PCI-to-PCI bridge device.  If so,
// and it's secondary bus matches the bus number for the target 
// device, I need to save the bridge's slot number.  If I can't 
// find an entry for the target device, I will have to assume it's 
// on the other side of the bridge, and assign it the bridge's slot.
			struct pci_dev*	pdev2 = pci_find_slot(tbus, tdevfn);
			if (pdev2 == NULL) continue;

			pci_read_config_dword(pdev2, 0x08, &work);

#ifdef DEBUG
			printk(KERN_DEBUG "class %8.8x vs PCI BRIDGE %8.8x\n", work>>8, PCI_TO_PCI_BRIDGE_CLASS);
#endif
			if ((work >> 8) == PCI_TO_PCI_BRIDGE_CLASS) {
				pci_read_config_dword(pdev2, 0x18, &work);
				// See if bridge's secondary bus matches target bus.
#ifdef DEBUG
				printk(KERN_DEBUG "secondary bus %d\n", (work>>8) & 0xff);
#endif

				if (((work>>8) & 0xFF) == (long) pdev->bus->number) {
					bridge_slot = tslot;
#ifdef DEBUG
					printk(KERN_DEBUG "match!\n");
#endif
				}
			}
		}

	}


	// If we got here, we didn't find an entry in the IRQ mapping table 
	// for the target PCI device.  If we did determine that the target 
	// device is on the other side of a PCI-to-PCI bridge, return the 
	// slot number for the bridge.
	if (bridge_slot != 0xFF) {
		*slot = bridge_slot;
		rc = 0;
		goto out;
	}

out:
	if (pci_irq_routing_info_length != NULL) kfree(pci_irq_routing_info_length );
	return (rc);
}
