#!/bin/bash
#
# setup.sh - Install the ProLiant Integration Module for Altiris eXpress on an NFS Server
#
# SAwe 09/30/02 - Added emtpy dir checks and return value to not copy OS files.
# SAwe 09/27/02 - Reversed "OS file are missing... Do you want to install OS" questions.
# SAwe 09/27/02 - Changed SUSE echo message from / to \ slashes when refering to Windows system.
# SAwe 09/11/02 - Added echo statements before removing directories.
# SAwe 08/27/02 - Added new RH80CopyDistribution function to account for new RH8.0 file naming convention.
# SAwe 08/26/02 - Added Red Hat Linux 8.0 to menu arrays and function call.  Removed RH7.3 references.

# the first n - 2 elements in menuselections array must have a
# corresponding element in installfunctions array
menuselections=("Red Hat Linux 8.0" "Red Hat Linux Advanced Server 2.1" "SuSE Linux Enterprise Server 7" "All Distributions" "Exit")
installfunctions=("InstallRedHatLinux80" "InstallRedHatLinuxAdvancedServer21" "InstallSuSELinuxEnterpriseServer7")
cdroot=.
srcroot=.
destroot=.
ss=ss.000
dsserver=0.0.0.0
nfsserver=0.0.0.0

###############################################################################

function CheckDiskSpace
{
   let needed=$1
   let free=$(df /usr | grep "dev" | awk '{print $4}')
   if [ $needed -ge $free ]; then
      echo
      echo "ERROR: There is not enough free disk space on /usr."
      echo
      exit 60
   fi
   return 0
}


function GetNFSServer
{
   declare -a addresses
   let index=0

   # HACK: SuSE scripted installs can't use the hostname
   if [ $(echo $menuselectionstring | cut -d' ' -f1) != "SuSE" -a $(echo $menuselectionstring | cut -d' ' -f1) != "All" ]; then

      # get hostname
      hostname=$(hostname -f 2>/dev/null)
      if [ -n "$hostname" -a "$hostname" != "localhost.localdomain" -a "$hostname" != "localhost" -a ! ${hostname%*.*} ]; then
         addresses[$index]=$hostname
         let index=$index+1
      fi

   fi

   # get ip addresses
   for i in $(ifconfig | grep "inet addr" | cut -d: -f2 | cut -d' ' -f1); do
      if [ "$i" != "127.0.0.1" ]; then
         addresses[$index]=$i
         let index=$index+1
      fi
   done

   # if only one entry, use it if it is an ip address
   if [ "$index" = "1" ]; then
      if [ ${index%[0-9]*.[0-9]*.[0-9]*.[0-9*} ]; then
         nfsserver=${addresses[0]}
         return 0
      else
         echo
         echo "ERROR: The network configuration of this machine is not correct."
         echo
         exit 60
      fi
   fi

   # select address to use
   echo
   echo "Select the hostname or IP address that you wish to use to identify the"
   echo "NFS server on the network. The target servers must have a route to the"
   echo "NFS server via the selected hostname or IP address."
   PS3='Address? '
   select addressselectionstring in "${addresses[@]}"; do
      let addressselection=$REPLY-1
      nfsserver=${addresses[$addressselection]}
      break
   done

   return 0
}


function AddNFSExport
{
   if ! grep -q $1 /etc/exports; then
      echo $1 \*\(ro\) >> /etc/exports
   fi
   return 0
}


function CopySupportFiles
{
   srcdir=$1
   destdir=$2

   # is the destination directory present
   dircnt=`ls -alq $destdir 2>>/dev/null | wc -l`
   if [ -d $destdir ] && [ $dircnt != 3 ]; then
      echo
      echo "The Compaq support files ($destdir) already exist."
      echo -n "Would you like to overwrite the existing files (Y/N)? "
      read prompt
      if [ ! ${prompt%[nN]*} ]; then
         return 1
      fi
      echo
      echo -n "Removing Compaq support files and directory..."
      rm -r $destdir
      echo "Done!"
   fi

   # check disk space
   CheckDiskSpace 10000

   # if the source is a CD, then prompt for it (Note: in the All case, the RDP CD will not be in the drive after the first distribution)
   if [ ${srcdir:0:1} != "." ]; then
      mount $cdroot &> /dev/null
      until [ -d $srcdir ]; do
         umount $cdroot &> /dev/null
         eject
         echo
         echo "Please insert the Rapid Deployment Pack CD."
         echo -n "(Press Enter to continue)"
         read answer
         mount $cdroot &> /dev/null
      done
   fi

   # copy
   mkdir -p $destdir
   echo
   echo -n "Copying support files..."
   cp -rfp $srcdir/* $destdir
   echo "Done!"

   # add $destdir to /etc/exports
   AddNFSExport $destdir

   return 0
}


function UpdateConfigurationFiles
{
   srcdir=$1
   filespec=$2

   cd $srcdir
   for file in $srcdir/$filespec; do
      chmod +w $file
      sed -e "s/~DSSERVER~/$dsserver/g" -e "s/~NFSSERVER~/$nfsserver/g" -e "s/~SS~/$ss/g" $file > $file.sav
      mv $file.sav $file
   done
   cd -
   return 0
}


function GenerateInitrdImages
{
   srcdir=$1

   cd $srcdir
   ./makeall.sh
   cd -
   return 0
}


#
# Used for all OS's except RH8.0.  RH80CopyDistribution is for RH8.0.
#
function CopyDistribution
{
   srcdir=$1
   destdir=$2
   distname=$3
   distcdcount=$4
   distkeyfile=$5
   distmagic=$6

   # if the destination directory is not present, let them skip
   dircnt=`ls -alq $destdir 2>>/dev/null | wc -l`
   if [ ! -d $destdir ] || [ $dircnt == 3 ]; then
      echo
      echo "The $distname files ($destdir) are not present."
      echo -n "Would you like to copy the files now (Y/N)? "
      read prompt
      if [ ! ${prompt%[nN]*} ]; then
         cat << CopyDistributionMessage

-----------------------------------------------------------------------------
IMPORTANT! You have chosen to not copy the $distname files now.
When manually copying the files, you must copy the entire contents of
$distname CDs 1 - $distcdcount to $destdir.

For detailed instructions, please refer to Chapter 2 in the ProLiant
Integration Module User Guide.
-----------------------------------------------------------------------------
CopyDistributionMessage
         echo -n "(Press Enter to continue)"
         read answer
         return 1
      fi
   fi

   # if the destination directory is present, let them overwrite
   dircnt=`ls -alq $destdir 2>>/dev/null | wc -l`
   if [ -d $destdir ] && [ $dircnt != 3 ]; then
      echo
      echo "The $distname files ($destdir) already exist."
      echo -n "Would you like to overwrite the existing files (Y/N)? "
      read prompt
      if [ ! ${prompt%[nN]*} ]; then
         return 1
      fi
      echo
      echo -n "Removing $distname files and directory..."
      rm -r $destdir
      echo "Done!"
   fi

   # check disk space
   CheckDiskSpace $((distcdcount * 650000))

   # copy
   mkdir -p $destdir
   echo
   let cd=1
   while [ $cd -le $distcdcount ]; do
      # get the correct cd
      keyfile=${distkeyfile/~/$cd}  #$(echo $distkeyfile | sed -e "s/~/$cd/g")
      mount $srcdir &> /dev/null
      until [ -e $srcdir/$keyfile ] && grep -q "$distmagic" $srcdir/$keyfile; do
         umount $srcdir &> /dev/null
         eject
         echo
         echo "Please insert $distname CD #$cd."
         echo -n "(Press Enter to continue)"
         read answer
         mount $srcdir &> /dev/null
      done
      
      # copy
      echo
      echo -n "Copying distribution..."
      cp -rfp $srcdir/* $destdir
      echo "Done!"
   
      let cd=$cd+1
   done
   umount $srcdir
   eject

   # add $destdir to /etc/exports
   AddNFSExport $destdir

   return 0
}


# 
# RH80 has new file naming convention.  This function is used solely
# for copying RH80 CD files.  All other OS's use generic CopyDistribution
# function.
#
function RH80CopyDistribution
{
   srcdir=$1
   destdir=$2
   distname=$3
   distcdcount=$4
   distkeyfile=$5
   distmagic=$6

   # if the destination directory is not present, let them skip
   dircnt=`ls -alq $destdir 2>>/dev/null | wc -l`
   if [ ! -d $destdir ] || [ $dircnt == 3 ]; then
      echo
      echo "The $distname files ($destdir) are not present."
      echo -n "Would you like to copy the files now (Y/N)? "
      read prompt
      if [ ! ${prompt%[nN]*} ]; then
         cat << RH80CopyDistributionMessage

-----------------------------------------------------------------------------
IMPORTANT! You have chosen to not copy the $distname files now.
When manually copying the files, you must copy the entire contents of
$distname CDs 1 - $distcdcount to $destdir.

For detailed instructions, please refer to Chapter 2 in the ProLiant
Integration Module User Guide.
-----------------------------------------------------------------------------
RH80CopyDistributionMessage
         echo -n "(Press Enter to continue)"
         read answer
         return 1
      fi
   fi

   # if the destination directory is present, let them overwrite
   dircnt=`ls -alq $destdir 2>>/dev/null | wc -l`
   if [ -d $destdir ] && [ $dircnt != 3 ]; then
      echo
      echo "The $distname files ($destdir) already exist."
      echo -n "Would you like to overwrite the existing files (Y/N)? "
      read prompt
      if [ ! ${prompt%[nN]*} ]; then
         return 1
      fi
      echo
      echo -n "Removing $distname files and directory..."
      rm -r $destdir
      echo "Done!"
   fi

   # check disk space
   CheckDiskSpace $((distcdcount * 650000))

   # copy
   mkdir -p $destdir
   echo
   let cd=1
   keyfile=$distkeyfile
   while [ $cd -le $distcdcount ]; do
      # get the correct cd
      mount $srcdir &> /dev/null
      cditem=""
      until [ -e $srcdir/$keyfile ] && grep -q "$distmagic" $srcdir/$keyfile && [ "$cditem" == "$cd" ]; do
         umount $srcdir &> /dev/null
         eject
         echo
         echo "Please insert $distname CD #$cd."
         echo -n "(Press Enter to continue)"
         read answer
         mount $srcdir &> /dev/null

         if [ -e $srcdir/$keyfile ]; then
           # parse the contents of keyfile, 4th row -- 5th item is CD number
           i=1
           for item in $(cat $srcdir/$keyfile); do
             if [ $i == 5 ]; then
               cditem=$item
               fi
             let i=$i+1
           done
         fi
      done
      
      # copy
      echo
      echo -n "Copying distribution..."
      cp -rfp $srcdir/* $destdir
      echo "Done!"
   
      let cd=$cd+1
   done
   umount $srcdir
   eject

   # add $destdir to /etc/exports
   AddNFSExport $destdir

   return 0
}


function RestartNFS
{
   # update configuration
   if grep -q "Red Hat" /etc/issue; then
      chkconfig --level 35 nfs on
   elif grep -q "SuSE" /etc/issue; then
      if grep -q -i NFS_SERVER=\"no\" /etc/rc.config; then
         chmod +w /etc/rc.config
         sed -e "s/NFS_SERVER=\"no\"/NFS_SERVER=\"yes\"/" /etc/rc.config > /etc/rc.config.sav
         mv /etc/rc.config.sav /etc/rc.config
         chmod -w /etc/rc.config
         SuSEconfig
      fi
   else
      echo
      echo "WARNING: Could not find an NFS server on this machine."
      echo -n "(Press Enter to continue)"
      return 1
   fi

   # restart nfs daemon
   /etc/init.d/nfs stop
   exportfs -r
   /etc/init.d/nfs start

   # check to see if it is running
   if ! ps -A | grep -q "nfs"; then
      echo
      echo "WARNING: Could not start the NFS server on this machine."
      echo -n "(Press Enter to continue)"
      read answer
   fi

   # check for a firewall
   if ipchains --list 2> /dev/null | grep -v -q "^Chain \w*"; then
      echo
      echo "WARNING: The firewall configuration may conflict with the NFS server."
      echo -n "(Press Enter to continue)"
      read answer
   fi

   return 0
}

#######################################

function InstallRedHatLinux80
{
   # distribution specific values
   dist=rh80
   distname="Red Hat Linux 8.0"
   distcdcount=3                   # Actually are 4 CDs, but need 3?
   distkeyfile=".discinfo"      
   distmagic="1031690066.345824"   # the magic value in the key file

   echo
   echo "Installing support for $distname"

   # copy support files
   srcdir=$srcroot/$ss/$dist
   destdir=$destroot/$ss/$dist
   if CopySupportFiles $srcdir $destdir; then
      UpdateConfigurationFiles $destdir '*ks.cfg'
      #UpdateConfigurationFiles $destdir/csp '*.sh'
   fi

   # copy distribution
   srcdir=$cdroot
   destdir=$destroot/$dist
   RH80CopyDistribution $srcdir $destdir "$distname" $distcdcount $distkeyfile $distmagic

   # message regarding events
   cat << RedHatLinux80Message

-----------------------------------------------------------------------------
IMPORTANT! You must update the $distname scripted install events on
your Deployment Server to specify this NFS server ($nfsserver).

Edit the "Run Script - OS Install" task and make the following change.
From:  set nfsserver=0.0.0.0
To:    set nfserver=$nfsserver

For detailed instructions, please refer to Chapter 3 in the ProLiant
Integration Module User Guide.
-----------------------------------------------------------------------------
RedHatLinux80Message
   echo -n "(Press Enter to continue)"
   read answer
}


function InstallRedHatLinuxAdvancedServer21
{
   # distribution specific values
   dist=rhas21
   distname="Red Hat Linux Advanced Server 2.1"
   distcdcount=2
   distkeyfile=".disc~-i386"       # the ~ represents the disc number
   distmagic="1020464629.319525"   # the magic value in the key file

   echo
   echo "Installing support for $distname"

   # copy support files
   srcdir=$srcroot/$ss/$dist
   destdir=$destroot/$ss/$dist
   if CopySupportFiles $srcdir $destdir; then
      UpdateConfigurationFiles $destdir '*ks.cfg'
      #UpdateConfigurationFiles $destdir/csp '*.sh'
   fi

   # copy distribution
   srcdir=$cdroot
   destdir=$destroot/$dist
   CopyDistribution $srcdir $destdir "$distname" $distcdcount $distkeyfile $distmagic

   # message regarding events
   cat << RedHatLinuxAdvancedServer21Message

-----------------------------------------------------------------------------
IMPORTANT! You must update the $distname scripted
install events on your Deployment Server to specify this NFS server
($nfsserver).

Edit the "Run Script - OS Install" task and make the following change.
From:  set nfsserver=0.0.0.0
To:    set nfserver=$nfsserver

For detailed instructions, please refer to Chapter 3 in the ProLiant
Integration Module User Guide.
-----------------------------------------------------------------------------
RedHatLinuxAdvancedServer21Message
   echo -n "(Press Enter to continue)"
   read answer
}


function InstallSuSELinuxEnterpriseServer7
{
   # distribution specific values
   dist=sles7
   distname="SuSE Linux Enterprise Server 7"
   distcdcount=2
   distkeyfile=".S.u.S.E-disk-00~.2001082217"  # the ~ represents the disc number
   distmagic="SuSE-Linux-SLES-i386 Version 7"  # the magic value in the key file

   echo
   echo "Installing support for $distname"

   # copy support files
   srcdir=$srcroot/$ss/$dist
   destdir=$destroot/$ss/$dist
   if CopySupportFiles $srcdir $destdir; then
      UpdateConfigurationFiles $destdir/initrd '*post.sh'
      UpdateConfigurationFiles $destdir/initrd '*.info'
      GenerateInitrdImages $destdir/initrd
   fi

   # copy distribution
   srcdir=$cdroot
   destdir=$destroot/$dist
   if CopyDistribution $srcdir $destdir "$distname" $distcdcount $distkeyfile $distmagic; then
      # fix common.pkd file
      cd $destroot/$dist/suse/setup/descr
      cp common.pkd common.pkd.bak
      perl -pi -e "s/InstPath: *02/InstPath:      01/" common.pkd
      cd -
   fi

   # message regarding events
   cat << SuSELinuxEnterpriseServer7Message

-----------------------------------------------------------------------------
IMPORTANT! You must copy the initrd files (*.img) from this server to the
Deployment Server and modify the "ProLiant ML/DL Scripted Install for
SuSE Linux Enterprise 7" event to refer to the correct initrd file.

Copy all of the *.img files from $destroot/$ss/$dist/initrd
to the following directory on the Deployment Server
\Program Files\Altiris\eXpress\Deployment Server\deploy\cds\compaq\\$ss\\$dist\boot

In the above event, edit the "Run Script - OS Install" task and change the line
"set initrdfile=unknown.img" to the appropriate initrd file.

For detailed instructions, please refer to Chapter 3 in the ProLiant
Integration Module User Guide.
-----------------------------------------------------------------------------
SuSELinuxEnterpriseServer7Message
   echo -n "(Press Enter to continue)"
   read answer
}

###############################################################################

#
# If not root, then exit
#
if [ "$UID" != "0" ]; then
   echo
   echo "ERROR: You must be root to run this script."
   echo
   exit 60
fi

#
# Determine the CD root
#
if grep -q "Red Hat" /etc/issue; then
   cdroot=/mnt/cdrom
elif grep -q "SuSE" /etc/issue; then
   cdroot=/media/cdrom
else
   echo
   echo "ERROR: Could not find a CD-ROM mount point."
   echo
   exit 60
fi

#
# If pwd is on the CD, then exit
#
if [ "$(pwd | cut -b-${#cdroot})" = "$cdroot" ]; then
   echo
   echo "ERROR: You can't run this script directly from a local directory on the CD. Please change to a directory not on the CD and execute $cdroot/pim-nfs/setup.sh."
   echo
   exit 60
fi

#
# If running from CD, copy and spawn
#
D=`dirname "$0"`
B=`basename "$0"`
abs0="`cd \"$D\" 2>/dev/null && pwd || echo \"$D\"`/$B"
if [ "$(echo $abs0 | cut -b-${#cdroot})" = "$cdroot" ]; then
   cp -fp $abs0 /tmp/setup.sh
   chmod +w /tmp/setup.sh
   cd /tmp
   exec /tmp/setup.sh
   exit 0
fi

# NOTE: At this point we are guaranteed to be running from the hard drive (either ./ or /tmp)

echo
echo "ProLiant Integration Module for Altiris eXpress Install"

#
# Determine the source root
#
if [ -n "$(ls | grep '^ss\.[0-9]*')" ]; then
   srcroot=.
elif [ -d $cdroot/pim-nfs ]; then
   srcroot=$cdroot/pim-nfs
else
   echo
   echo "ERROR: Could not find the Rapid Deployment Pack files."
   echo
   exit 60
fi
ss=$(ls $srcroot | grep ss\.[0-9]*)

#
# Determine the destination root
#
destroot=/usr/cpqrdp
if [ ! -d $destroot ]; then
   mkdir -p $destroot
fi

#
# Choose distribution(s) to install
#
echo
echo "Select the distribution(s) that you wish to install via NFS."
PS3='Distribution? '
let maxmenuselections=${#installfunctions[@]}
select menuselectionstring in "${menuselections[@]}"; do
   let menuselection=$REPLY-1
   case $menuselection in
      [0-$(($maxmenuselections - 1))] )
         GetNFSServer
         ${installfunctions[$menuselection]}
         RestartNFS
         break;;
      $maxmenuselections )
         GetNFSServer
         for installfunction in "${installfunctions[@]}"; do
            $installfunction
         done
         RestartNFS
         break;;
   esac
   break
done
echo
